@@ -658,6 +658,219 @@ class ToroidalBoggleBoard(RectangularToroidalBoggleBoard):
 
 
 
+class RectangularHexagonalBoggleBoard(AbstractBoggleBoard):
+    """A Boggle board where instead of square tiles, we have hexagonal shaped
+    ones."""
+
+    def __init__(self, length=4, width=4, layout=None):
+        """__init__(length=4, width=4, layout=None)
+
+        Create a length by width Boggle board consisting of hexagonal tiles."""
+
+        self.width = width
+        self.length = length
+        AbstractBoggleBoard.__init__(self, length*width, layout)
+
+
+    def _determineAdjacencies(self):
+        """_determineAdjacencies()
+
+        Should not be called by the end user.
+        Create the adjacency graph for the board."""
+
+        adjacencies = [[] for i in range(self.numLetters)]
+
+        # Unlike the rectangular boggle board, which has layout:
+        #  0   1   2 ...
+        #  w   w+1 w+2 ...
+        #  etc...
+        #
+        # we instead first descend down columns and then across by rows:
+        # 0       2w
+        #    w         ...
+        # 1       2w+1
+        #    w+1       ...
+        # 2       2w+2
+        # .  w+2   .   ...
+        # .   .    .
+        # .   .    .
+        for i in range(self.numLetters):
+            # Try moving up. This is not possible if we are divisible by the length.
+            if i % self.length != 0:
+                adjacencies[i].append(i-1)
+ 
+            # Try moving down. This is not possible if we are 1 short of the length.
+            if (i+1) % self.length != 0:
+                adjacencies[i].append(i+1)
+
+            # Try moving left and up, and right and up. To do so, we must not be
+            # an even multiple of the length.
+            if i % (2*self.length) != 0:
+                # To move left and up, we cannot be in the first column.
+                if i >= self.length:
+                    # Even column: move -length-1 positions.
+                    # Odd column:  move -length positions.
+                    adjacencies[i].append(i - self.length - (1 if (int(i / self.length)) % 2 == 0 else 0))
+                # To move right and up, we cannot be in the last column.
+                    # Even column: move length-1 positions.
+                    # Odd column:  move length positions.
+                if i < self.numLetters - self.length:
+                    adjacencies[i].append(i + self.length - (1 if (int(i / self.length)) % 2 == 0 else 0))
+
+            # Try moving left and down, and right and down. To do so, we must
+            # not be 1 short of an even multiple of the length.
+            if (i+1) % (2*self.length) != 0:
+                # To move left and down, we cannot be in the first column.
+                if i >= self.length:
+                    # Even column: move -length positions.
+                    # Odd column:  move -length+1 positions.
+                    adjacencies[i].append(i - self.length + (int(i / self.length)) % 2)
+                # To move right and down, we cannot be in the last column.
+                if i < self.numLetters - self.length:
+                    # Even column: move length positions.
+                    # Odd column:  move length+1 positions.
+                    adjacencies[i].append(i + self.length + (int(i / self.length)) % 2)
+
+        return adjacencies
+
+
+    def __str__(self):
+        """__str__
+
+        Convert a hexagonal Boggle board to a readable string."""
+
+        if self.layout == None:
+            return ''
+            
+        # Determine the length of the longest object's string representation.
+        twidth = max([len(a.__str__()) for a in self.layout])+1
+        formatstr = '%%%ds' % twidth
+
+        # As we are hexagonal, we must modify spacing. We print all the even
+        # columns on one row, then all the odd columns on the following row,
+        # etc, as above in the comments for _determineAdjacencies.
+
+        strRep = ''
+        for i in range(self.length):
+            strRep += (formatstr * self.width + '\n') % tuple([self.layout[self.length*j+i].upper() if j%2 == 0 else '' for j in range(self.width)])
+            strRep += (formatstr * self.width + '\n') % tuple([self.layout[self.length*j+i].upper() if j%2 == 1 else '' for j in range(self.width)])
+        return strRep
+
+
+
+class HexagonalBoggleBoard(RectangularHexagonalBoggleBoard):
+    """A standard n x n Boggle board with hexagonal tiles. (Default size is 4.)"""
+
+    def __init__(self, size=4, layout=None):
+        """__init__(size=4, layout=None)
+
+        Create a standard size x size Boggle board with hexagonal tiles."""
+
+        RectangularBoggleBoard.__init__(self, size, size, layout)
+
+
+    def generateBoard(self, dice=defaultDice16, seed=-1):
+        """generateBoard(dice=defaultDice16, seed=-1)
+
+        Generate a boggle board as per the generateBoard method of
+        AbstractBoggleBoard using the specified dice set."""
+
+        RectangularBoggleBoard.generateBoard(self, dice, seed)
+
+
+
+class RectangularToroidalHexagonalBoggleBoard(RectangularHexagonalBoggleBoard):
+    """A boggle board on the surface of a torus, i.e. the letters on the left
+    side of the board are adjacent to those on the right side and vice versa,
+    and the letters on the top of the board are adjacent to those on the bottom
+    and vice versa. For example, if we have the 4x4 boggle board with entries
+    indicated by a positional marker:
+
+    0   8
+      4   C
+    1   9
+      5   D
+    2   A
+      6   E
+    3   B
+      7   F
+
+    Then note that we have that 0 is adjacent, as usual, to 1 and 4, but
+    furthermore, since the left side is adjacent to the right side, also
+    C, and since the top is adjacent to the bottom, also 3 and 7.
+    Additionally, note that 0 is adjacent to F as well, as it is adjacent
+    to both the right side and the bottom side.
+
+    This board is more equitable towards letters, as every letter in the board
+    is adjacent to exactly six neighbouring letters, unlike the standard
+    hexagonal boggle board where letters may be adjacent to 2, 3, 4, 5, or 6
+    letters.
+
+    NOTE: For the torus shape to work properly in the hexagonal case, we must
+    have an even number of columns."""
+
+    def __init__(self, length, width, layout=None):
+        """__init__(length, width, layout=None)
+
+        Create a rectangular hexagonal Boggle board on the surface of a torus.
+        Note that the width must be even for the connections to make sense."""
+
+        if width % 2 != 0:
+            raise ValueError("Width for toroidal hexagonal board must be even: %s" % width)
+        RectangularHexagonalBoggleBoard.__init__(self, length, width, layout)
+
+
+    def _determineAdjacencies(self):
+        """_determineAdjacencies
+
+        Create the adjacency graph to account for the left-right and top-bottom
+        looping."""
+
+        # Create a grid representing the board for modular arithmetic to
+        # simplify calculations.
+        grid = [[i * self.length + j for j in range(self.length)] for i in range(self.width)]
+
+        # Create the adjacency array of size self.length * self.width
+        adjacencies = [[] for i in range(self.numLetters)]
+
+        for (entry,(i,j)) in enumerate((i,j) for i in range(self.width) for j in range(self.length)):
+            # Move up and down.
+            adjacencies[entry].append(grid[i][(j-1) % self.length])
+            adjacencies[entry].append(grid[i][(j+1) % self.length])
+ 
+            # Move in all other directions.
+            # Even column: for col+1, col-1, we move row, row-1.
+            # Odd column:  for col+1, col-1, we move row, row+1.
+            for x in (-1,1):
+                adjacencies[entry].append(grid[(i+x) % self.width][j])
+                adjacencies[entry].append(grid[(i+x) % self.width][(j + (-1 if i % 2 == 0 else 1)) % self.length])
+
+        return adjacencies
+
+
+
+class ToroidalHexagonalBoggleBoard(RectangularToroidalHexagonalBoggleBoard):
+    """A standard (square) toroidal boggle board with hexagon tiles."""
+
+    def __init__(self, size=4, layout=None):
+        """__init__(size=4, layout=None)
+
+        Create a standard size x size toroidal hexagonal boggle board."""
+
+        self.size = size
+        RectangularToroidalHexagonalBoggleBoard.__init__(self, size, size, layout)
+
+
+    def generateBoard(self, dice=defaultDice16, seed=-1):
+        """generateBoard(dice=defaultDice16, seed=-1)
+
+        Generate a boggle board as per the generateBoard method of
+        AbstractBoggleBoard using the specified dice set."""
+
+        RectangularToroidalHexagonalBoggleBoard.generateBoard(self, dice, seed)
+
+
+
 class Anagram(AbstractBoggleBoard):
     """An anagram can be represented by a BoggleBoard where every letter is
     considered to be adjacent to every other letter."""
@@ -690,6 +903,17 @@ class Anagram(AbstractBoggleBoard):
 
 
 
+def createWordTrie(filename='words'):
+    """createWordTrie:
+
+    Create a trie from the specified file."""
+
+    infile = open(filename, 'r')
+    t = trie.Trie([a.strip() for a in infile.readlines()])
+    return t
+
+
+
 if __name__ == '__main__':
     if len(sys.argv) > 2:
         sys.stderr.write('Usage: %s [board-number]\n' % sys.argv[0])
@@ -706,14 +930,3 @@ if __name__ == '__main__':
     boggleBoard = BoggleBoard()
     board = boggleBoard.generateBoard(seed=seed)
     print boggleBoard
-
-
-
-def createWordTrie(filename='words'):
-    """createWordTrie:
-
-    Create a trie from the specified file."""
-
-    infile = open(filename, 'r')
-    t = trie.Trie([a.strip() for a in infile.readlines()])
-    return t
